# beaconadjacency.py -- tries to establish receiver adjacency by
# looking for identical sequence numbers received multiple times
# in 23C3 Sputnik log file
# (c) Peter Meerwald, pmeerw@pmeerw.net
#
# input format is text (either STDIN or files) with
# <timestamp> <IP> <sequence #>\n
#
# output to STDOUT is a graphviz dot file for plotting
#
# history:
#   0.1 initial version
#
# This file is distributed under GPL license.
#

import fileinput
import sys
import datetime

adjacency = {}

def adjacency_str(x):
	if x > 100000: return ', style=bold, len=1'
	else: return ''

def all_2tuples(list_in):
	list_out = []
	for e1 in list_in:
		for e2 in list_in:
			if e2 >= e1: continue
			if (e1, e2) not in list_out: list_out.append((e1, e2))
	return list_out
			
fi = fileinput.input(sys.argv[1:])

seqs = {}
lineno = 0
old_ts = ''
for line in fi:
	ts, ip, seqno = line.split()

	if ts != old_ts:
		for seq in seqs.keys():
			for tuple in all_2tuples(seqs[seq]):
				if adjacency.has_key(tuple): adjacency[tuple] += 1
				else: adjacency[tuple] = 1
				
		seqs = {}
		old_ts = ts
	
	if seqs.has_key(seqno):
		seqs[seqno].append(ip)
	else:
		seqs[seqno] = [ip]

	timestamp = datetime.datetime.fromtimestamp(float('0x'+ts))
	# print timestamp

	
	lineno += 1

	# for testing
	# if lineno > 10000:
	#	break

# -------------------------------------------------------

print 'graph IPs {'
print '  subgraph clusterB {'
print '    "10.254.0.254" [color=red];'
print '    "10.254.0.7" [color=red];'
print '    "10.254.1.12" [color=red];'
print '    "10.254.1.22" [color=red];'
print '    "10.254.1.5" [color=red];'
print '    "10.254.1.6" [color=red];'
print '    "10.254.1.16" [color=red];'
print '    "10.254.2.5" [color=red];'
print '    "10.254.3.21" [color=red];'
print '    "10.254.3.9" [color=red];'
print '    "10.254.4.12" [color=red];'
print '    "10.254.5.2" [color=red];'
print '    label="Floor B";'
print '  };'
print '  subgraph clusterA {'
print '    "10.254.1.7" [color=blue];'
print '    "10.254.2.10" [color=blue];'
print '    "10.254.2.12" [color=blue];'
print '    "10.254.2.3" [color=blue];'
print '    "10.254.3.5" [color=blue];'
print '    "10.254.3.13" [color=blue];'
print '    "10.254.4.6" [color=blue];'
print '    label="Floor A";'
print '  };'
print '  subgraph clusterC {'
print '    "10.254.4.11" [color=green];'
print '    "10.254.5.21" [color=green];'
print '    "10.254.7.14" [color=green];'
print '    "10.254.8.14" [color=green];'
print '    label="Floor C";'
print '  };'
for tuple in adjacency.keys():
	if adjacency[tuple] < 20000: continue
	print '  "%s" -- "%s" [ label="%d"%s];' % (tuple[0], tuple[1], adjacency[tuple], adjacency_str(adjacency[tuple]))
print '}'
