# beaconstat.py -- computes histogram data from 23C3 Sputnik log file
# (c) Peter Meerwald, pmeerw@pmeerw.net
#
# input format is text (either STDIN or files) with
# <timestamp> <IP>\n
#
# output is three files:
# 1. ip.txt with <IP> <# trackings>\n
# 2. minute.txt with <YYYY-MM-DD HH:MM> <# trackings>\n
# 3. minute-<IP>.txt with <YYYY-MM-DD HH:MM> <# trackings>\n
#
# history:
#   0.1 initial version
#
# This file is distributed under GPL license.
#

import fileinput
import sys
import datetime

fi = fileinput.input(sys.argv[1:])

ip_dict = {}
timestamp_dict = {}
ip_timestamp_dict = {}


lineno = 0
for line in fi:
	timestamp, ip = line.split()

	timestamp = datetime.datetime.fromtimestamp(float('0x'+timestamp))	
	timestamp_str = '%d-%02d-%02d %02d:%02d' % (timestamp.year, timestamp.month, timestamp.day, timestamp.hour, timestamp.minute)
	if timestamp_dict.has_key(timestamp_str):
		timestamp_dict[timestamp_str] += 1
	else:
		timestamp_dict[timestamp_str] = 1

	if ip_dict.has_key(ip):
		ip_dict[ip] += 1
	else:
		ip_dict[ip] = 1

	if ip_timestamp_dict.has_key(ip):
		if ip_timestamp_dict[ip].has_key(timestamp_str):
			ip_timestamp_dict[ip][timestamp_str] += 1
		else:
			ip_timestamp_dict[ip][timestamp_str] = 1
	else:
		ip_timestamp_dict[ip] = {timestamp_str:1}
	
	lineno += 1

	# for testing
	# if lineno > 10000:
	# 	break


# -------------------------------------------------------		
# output histograms

# output trackings per IP
fip = open('ip.txt', 'wt')
for ip in ip_dict.keys():
	print >>fip, '%s %d' % (ip, ip_dict[ip])
fip.close()

# output trackings per minute
sorted_timestamps = timestamp_dict.keys()
sorted_timestamps.sort()

fts = open('minute.txt', 'wt')
for timestamp in sorted_timestamps:
	print >>fts, '%s %d' % (timestamp, timestamp_dict[timestamp])
fts.close()

# output trackings per IP per minute
for ip in ip_timestamp_dict.keys():
	fipts = open('minute-%s.txt' % (ip), 'wt')
	sorted_timestamps = ip_timestamp_dict[ip].keys()
	sorted_timestamps.sort()
	for timestamp in sorted_timestamps:
		print >>fipts, '%s %d' % (timestamp, ip_timestamp_dict[ip][timestamp])

	fipts.close()
